abclinuxu.cz AbcLinuxu.cz itbiz.cz ITBiz.cz HDmag.cz HDmag.cz abcprace.cz AbcPráce.cz
AbcLinuxu hledá autory!
Inzerujte na AbcPráce.cz od 950 Kč
Rozšířené hledání
×
    dnes 13:44 | Nová verze

    Desktopové prostředí KDE Plasma bylo vydáno ve verzi 6.8. Novinky zahrnují lepší podporu vzdálené plochy, sdílení a nahrávání obrazovky a řadu drobností napříč uživatelským rozhraním (úplný seznam změn, shrnutí ve wiki).

    ʚĭɞ 🍉 | Komentářů: 4
    dnes 10:55 | IT novinky

    Apple představil 7palcový displej Apple HomeView s podporou Apple Intelligence a Siri AI pro ovládání chytré domácnosti, novou řadu vybavení chytré domácnosti Apple Home Collection, další generaci HomePodu mini a novou Apple TV 4K. Apple HomeView není v Evropské unii v prodeji, jelikož umělá inteligence Siri v rámci systému iOS v EU momentálně není k dispozici. Apple spolupracuje s EU a snaží se najít cestu, jak situaci vyřešit.

    Ladislav Hagara | Komentářů: 2
    dnes 03:55 | Komunita

    KDE slaví 30 let. Přesně před 30 lety, 14. října 1996, poslal Matthias Ettrich do diskusní skupiny comp.os.linux.misc zprávu, která započala historii projektu KDE. Důležité milníky jsou zobrazeny na časové ose.

    Ladislav Hagara | Komentářů: 3
    dnes 03:22 | Nová verze

    Byl vydán Mozilla Firefox 158.0. Stejně jako Chrome 155 s podporou JPEG XL (test). Přehled novinek v poznámkách k vydání a poznámkách k vydání pro vývojáře. Řešeny jsou rovněž bezpečnostní chyby. Nový Firefox 158 bude brzy k dispozici také na Flathubu a Snapcraftu.

    Ladislav Hagara | Komentářů: 0
    dnes 03:00 | Nová verze

    Svobodný kancelářský balík ONLYOFFICE (Wikipedie) byl vydán ve verzi 10.0. Jak online Docs, tak i offline Desktop Editors. Přehled novinek také na YouTube.

    Ladislav Hagara | Komentářů: 0
    včera 14:44 | IT novinky

    Mozilla představuje Firefox Enterprise určený jako podnikový prohlížeč pro organizace s přísnými požadavky na bezpečnost, správu a dodržování předpisů. S profesionální podporou. Dostupný je zatím ve Francii, Německu, Rakousku a Švýcarsku.

    Ladislav Hagara | Komentářů: 11
    včera 14:33 | Bezpečnostní upozornění

    Kybernetický útok zasáhl nejméně 150 supermarketů Intermarché a několik prodejen řetězců Leclerc a Coopérative U. Chladicí jednotky napadených obchodních domů byly nastaveny na nepřetržité odmrazování a teplota v nich mnohdy vystouala až nad 50°C. Škody napáchané na chlazeném a mraženém zboží budou patrně dosahovat miliónů eur.

    … více »
    ❗Červivý Hřib krade❗ | Komentářů: 3
    včera 04:00 | Komunita

    Stát Kalifornie zažaloval výrobce 3D tiskových filamentů Polymaker. Vrchní soud San Diego County tvrdí, že firma propagací svých výrobků mezi příznivci 3D tištěných zbraní podporovala výrobu takzvaných 'ghost guns' (DIY zbraní) a že její PLA filament pro 3D tiskárny lze za těchto okolností považovat za součást určenou k výrobě střelné zbraně. Žaloba požaduje mimo jiné finanční sankce a soudní zákaz činnosti, Polymaker se hájí tím, že filament je běžný a sám o sobě legální produkt. Celý text žaloby (PDF na Google Drive).

    ❗Červivý Hřib krade❗ | Komentářů: 26
    včera 03:33 | Nová verze

    V červnu byla představena aplikace Myna pro převod řeči na text v prostředí desktopu Ubuntu. Včera byla Myna zpřístupněna širší komunitě Ubuntu formou technologické ukázky dostupné v Ubuntu 24.04 LTS, Ubuntu 26.04 LTS a Ubuntu 26.10 s GNOME na Waylandu.

    Ladislav Hagara | Komentářů: 0
    12.10. 21:11 | Komunita

    Ubuntu 27.04 bude Tenacious T-Rex.

    Ladislav Hagara | Komentářů: 0
    Které desktopové prostředí na Linuxu používáte?
     (9%)
     (7%)
     (4%)
     (21%)
     (29%)
     (8%)
     (5%)
     (2%)
     (14%)
     (20%)
    Celkem 2844 hlasů
     Komentářů: 32, poslední 11.10. 17:02
    Rozcestník

    Dotaz: Jazyk C: multiple definition of id

    31.10.2008 22:57 Alexander Krupka
    Jazyk C: multiple definition of id
    Přečteno: 2835×
    Dobry den,

    napsal jsem program, ktery se sestava z techto souboru:
    Makefile
    scanner-test.c
    scanner.c
    scanner.h
    str.c
    str.h
    
    a pri prekladu pres make mi to stale haze toto:
    $ make
    gcc -Wall -std=c99 -pedantic -lm -c scanner-test.c
    gcc -Wall -std=c99 -pedantic -lm -c scanner.c
    gcc -Wall -std=c99 -pedantic -lm -c str.c
    gcc -Wall -std=c99 -pedantic -lm scanner-test.o scanner.o str.o -o scanner-test
    scanner.o:(.data+0x0): multiple definition of `keywords'
    scanner-test.o:(.data+0x0): first defined here
    collect2: ld returned 1 exit status
    make: *** [scanner-test] Error 1
    
    keywords je pole, ktere je definovane POUZE jednou v souboru scanner.h. Mam podezreni, ze za toto muze muj Makefile. Ten vypada takto:
    CC=gcc
    CFLAGS=-Wall -std=c99 -pedantic -lm
    
    all: scanner-test
    
    scanner-test: scanner-test.o scanner.o str.o
    	$(CC) $(CFLAGS) scanner-test.o scanner.o str.o -o scanner-test
    
    scanner.o: scanner.c str.h
    	$(CC) $(CFLAGS) -c scanner.c 
    
    str.o: str.c str.h
    	$(CC) $(CFLAGS) -c str.c
    
    scanner-test.o: scanner-test.c str.h scanner.h
    	$(CC) $(CFLAGS) -c scanner-test.c
    
    clean:
    	rm -f *.o *.out $(PRJ)
    
    Nezda se mi vsak, ze by se tam neco natahovalo 2x. Uz jsem z toho zoufaly, hledani teto priciny mi uz zabralo vice casu nez samotne napsani programu, muzete mi prosim poradit? Dekuji.

    Odpovědi

    31.10.2008 23:09 petr
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Je to tim, ze scanner.h mate naincludovany i do scanner.c. To neni problem v Makefilu, proste mate blbe kod. Najdete si na netu nebo v jakekoliv ucebnici C klicova slova extern a static.
    31.10.2008 23:10 Alexander Krupka
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Jeste dodam, ze jsem zkusil zmenit Makefile na tento, ale stale stejny vysledek:
    CC=gcc
    CFLAGS=-Wall -std=c99 -pedantic -lm
    
    all: str.o scanner.o scanner-test.o
    	$(CC) $(CFLAGS) -o scanner-test str.o scanner.o scanner-test.o
    
    clean:
    	rm -f *.o *.out $(PRJ)
    
    Jeste popisu co jednotlive souboru includuji:
    scanner.c     <-- scanner.h
                      str.h
    
    str.c         <-- str.h
    
    scanner-test  <-- str.h
                      scanner.h
    
    Vsechny hlavickove soubory jsou opatreny konstrukci
    #ifndef XXX
    #define XXX
    ...
    #endif
    
    31.10.2008 23:12 XSimi | skóre: 10
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    mas .h v nejakym takovymdle tvaru?
    
    #ifndef TESTYTESTY_H
    #define TESTYTESTY_H
    
    cosi
    
    #endif
    
    31.10.2008 23:16 XSimi | skóre: 10
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    ups slepej sem
    1.11.2008 08:20 burlog | skóre: 9
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Toto je sice take potreba, ale tazatel potrebuje skutecne danou promenou udelat extern:
    a.h
    ----
    
    extern int a;
    
    a.c
    ----
    
    int a = 0;
    
    pak to zacne fungovat.

    1.11.2008 10:21 Alexander Krupka
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Tak jsem pole keywords definoval jak extern (extern const char*):
    $ make
    gcc -Wall -std=c99 -pedantic -lm   -c -o str.o str.c
    gcc -Wall -std=c99 -pedantic -lm   -c -o scanner.o scanner.c
    In file included from scanner.c:5:
    scanner.h:12: warning: ‘keywords’ initialized and declared ‘extern’
    gcc -Wall -std=c99 -pedantic -lm   -c -o scanner-test.o scanner-test.c
    In file included from scanner-test.c:4:
    scanner.h:12: warning: ‘keywords’ initialized and declared ‘extern’
    gcc -Wall -std=c99 -pedantic -lm -o scanner-test str.o scanner.o scanner-test.o
    scanner-test.o:(.data+0x0): multiple definition of `keywords'
    scanner.o:(.data+0x0): first defined here
    collect2: ld returned 1 exit status
    make: *** [all] Error 1
    
    Nepomohlo:/
    1.11.2008 10:28 Alexander Krupka
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Jeste dodam, ze promenna keywords je definovana v souboru scanner.h a jeji jedine pouziti je v souboru scanner.c.
    1.11.2008 10:43 petr
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Pokud to pouzivate jen v jednom souboru, tak to nadeklarujte jako static.
    1.11.2008 11:01 Alexander Krupka
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    scanner.h:
    static const char* keywords[] = {
        ...
    };
    
    make:
    $ make
    gcc -Wall -std=c99 -pedantic -lm   -c -o str.o str.c
    gcc -Wall -std=c99 -pedantic -lm   -c -o scanner.o scanner.c
    gcc -Wall -std=c99 -pedantic -lm   -c -o scanner-test.o scanner-test.c
    scanner.h:12: warning: ‘keywords’ defined but not used
    gcc -Wall -std=c99 -pedantic -lm -o scanner-test str.o scanner.o scanner-test.o
    
    Nejak to ale nechapu, proc jsem musel promennou deklarovat jako static? Jako deklarace te promenne je v souboru scanner.h. Jeji pouziti je vsak ve scanner.c. Jaktoze si tedy scanner.c nestezuje, ze nevidi definici teto promenne?
    1.11.2008 11:11 petr
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Nasel jste si v nejake knizce nebo na netu to extern a static, jak jsem vam hned na zacatku napsal? Prijde mi zbytecne sem psat neco, co najdete v jakekoliv ucebnici C, takze ve zkratce 3 druhy deklarace a definice:

    auto - kde je definovana, tak tam se vytvori, globalni viditelnost. static - kde je definovana, tam se vytvori, viditelnost v aktualnim modulu. extern - deklarace rika, ze pozdeji bude definovana promenna toho jmena v jinem modulu.

    1.11.2008 11:39 Alexander Krupka
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Ano, toto jsem si zjistil, ale nevim jak toho vyuzit k vyreseni meho problemu.
    3.11.2008 06:36 pht | skóre: 48 | blog: pht
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    kdyz to pouzivate jen v jednom .c tak proc to cpete do .h ktere pak inkludujete vsude mozne?
    In Ada the typical infinite loop would normally be terminated by detonation.
    1.11.2008 10:42 petr
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Protoze to mate spatne. Extern je smysluplne jen v deklaraci, nikoliv v definici.
    1.11.2008 10:57 luky
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Mate zmatek v rozdilu mezi definici a deklaraci. Kdyz mate v .h definici promenne, tak bude definovana v kazdem souboru, ktery ho inkluduje (trik s include-once (ifdefy) vam nepomuze, protoze se kazdy .c kompiluje zvlast).
    1.11.2008 11:10 Alexander Krupka
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    int a; ... definice int a = 5; deklarace

    ... je to tak spravne?

    Jake reseni tedy z toho vaseho prostrehy vyplyva? Nechce se mi deklaraci nebo co to je, proste toto:
    const char* keywords[] = {
        ...
    };
    
    presouvat ze scanner.h do scanner.c, vzdyt od toho mimo jine scanner.h, je? Na deklaraci promennych a typu, ktere pak pouziva *.c soubor.
    1.11.2008 13:01 Michal Kubeček | skóre: 71 | Luštěnice
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id

    Když napíšete

      int a;
      int b = 5;
    

    tak obě proměnné zároveň definujete, tj. v příslušném modulu bude tato proměnná vytvořena (jestli tam máte nebo nemáte inicializaci, nehraje roli). Napíšete-li takovou věc do hlavičkového souboru, proměnná se vytvoří v každém modulu, do kterého ho includujete, takže linker dostane kolizi. Když ale napíšete

      extern int a;
      extern int b;
    

    řeknete tím překladači, že proměnné a, b jsou typu int, ale že je v tomto modulu nemá vytvářet, protože jsou definovány jinde (tj. externě). První příklad odpovídá definici funkce (tj. včetně těla), zatímco druhý její deklaraci (jen hlavička).

    Takže správný postup, jak už tu padlo, je

      extern const char* keywords[];
    

    ve scanner.h a

      extern const char* keywords[] = {
        ...
      };
    

    ve scanner.c, čímž zajistíte, že ve všech modulech (kde se includuje scanner.h) budete mít proměnnou deklarovanou, ale vytvoří se jen ve scanner.o, takže linker na žádnou kolizi nenarazí. Použití static je v tomto případě chybné, protože to by znamenalo, že to sice kolidovat nebude, ale jen proto, že každý modul bude mít svou vlastní proměnnou stejného jména.

    2.11.2008 21:37 Sinuhet | skóre: 31
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Chybycka se vam tam vloudila, scanner.c ma byt
    const char* keywords[] = { // bez extern
        ...
    };
    
    jinak to bude kolegovi pri linkovani pro zmenu rvat, ze keywords neexistuje.
    2.11.2008 22:31 Michal Kubeček | skóre: 71 | Luštěnice
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    Díky za opravu. Klasická copy-and-paste chyba…
    1.11.2008 19:04 Martin Tůma | skóre: 39 | blog: RTFM | Praha
    Rozbalit Rozbalit vše Re: Jazyk C: multiple definition of id
    vzdyt od toho mimo jine scanner.h, je? Na deklaraci promennych a typu, ktere pak pouziva *.c soubor.

    Ne tak docela. Hlavičkový soubor slouží především k deklaraci funkcí/proměnných které využívají jiné moduly. Pokud, jak zde píšete, proměnnou/datový typ používáte pouze a jenom v *.c souboru příslušném k danému *.h souboru, je více než moudré proměnnou v hlavičkovém souboru neuvádět a navíc jí deklarovat jako static. Předejdete tak problémům se stejně pojmenovanými "lokálními" funkcemi/proměnnými a "zpřehledníte" informaci o tom, co vlastně daný modul exportuje.

    Každý má právo na můj názor!

    Založit nové vlákno • Nahoru

    Tiskni Sdílej: Linkuj Jaggni to Vybrali.sme.sk Google Del.icio.us Facebook

    ISSN 1214-1267   www.czech-server.cz
    © 1999-2015 Nitemedia s. r. o. Všechna práva vyhrazena.